if not srb2p return end

freeslot("SPR_WHCY", "sfx_whrl", "sfx_dnts", "sfx_dntg", "sfx_dntr", "sfx_dntt", "sfx_dntw", "SPR_PSYC")
//Wheel mechanic

if not T_WispTable
	rawset(_G, "T_WispTable", {})
end

/*
Howdy. If you're looking at this, you're probably curious as to how all this garbage works. Well, let me tell you. 
This file handles all the Wisp Change stuff that isn't a move or stat definitons.

The basis for a wisp is that it has a set of skills and passives that get exchanged for the other wisp's counterparts, as well as a set
of stat changes the Wisp provides and a starting color starting from the first defined color.

However, this system has a few limits. First, all you cannot have any more or less skills than as provided by the default wisp skills, and
the order has to be specific. So first, let me give you a list of what skills are what.

Wispon Attack - The basic attack attached to the Wispon.

Wispon Amplifier - A three-tier set of skills that, at least in the case of the original wisps, are strong attacks with a cooldown attached.

Wispon Support - A set of three skills. The first skill is learned early, but branches off into two variants that are not necessarily better than one another.

Wispon Ability - A set of two skills, with the later version better than the first.

Wispon Passive - A set of three passives attached to the wisp. 

The order these go in is: {Wispon Attack, Wispon Amp 1, Wispon Amp 2, Wispon Amp 3, Wispon Support 1, Wispon Support AoE, Wispon Support Strong, Wispon Ability 1, Wispon Ability 2}
The order is important as it's how the game decides which skills to replace with what. The passives use a seperate handler, so do not slot normal skills as passives.
*/

do
	local ogWisps = {
		{
			name = "Laser", -- This is purely for identification's sake.
			icon = "CYLCH1",
			startcolor = SKINCOLOR_SUPERLASER1, -- The starting color for the Wisp to use.
			skills = {"whisper_atkl", "tracer round 1", "tracer round 2", "tracer round 3", "w-tankbreaker", "w-equalize", "w-devastate", "w-miranda", "w-mamiranda"}, -- The skill name list
			passives = {"laser focus 1", "laser focus 2", "laser focus 3"}, -- The passive name list
			passiveStats = {
				{5, 5, 10, 5, 0},
				{10, 10, 20, 10, 0},
				{15, 15, 35, 15, 0}} -- The passive bundles. The first 5 arguments are percentage changes to Strength, Defense, Agility, Magic, and Luck in that order. i.e. 60 is a 60% buff.
		},
		{
			name = "Rocket",
			icon = "CYLCH2",
			startcolor = SKINCOLOR_SUPERROCKET1,
			skills = {"whisper_atkr", "big bang 1", "big bang 2", "big bang 3", "w-makanda", "w-mamakanda", "w-debilitate", "w-deep breath", "w-zen mindset"},
			passives = {"extra powder 1", "extra powder 2", "extra powder 3"},
			passiveStats = {
				{0, -8, -8, 15, 0, "recoil", 5}, -- You can also pass a 6 or more arguments. The 6th argument decides the effect, while the following arguments determine the arguments for that effect.
				{0, -16, -16, 30, 0, "recoil", 10},
				{0, -24, -24, 50, 0, "recoil", 20}}
		},
		{
			name = "Hover",
			icon = "CYLCH3",
			startcolor = SKINCOLOR_SUPERHOVER1,
			skills = {"whisper_atkh", "plasma burst 1", "plasma burst 2", "plasma burst 3", "w-sukunda", "w-masukunda", "w-accelerate",  "w-airlift", "w-abduction"},
			passives = {"lightweight boots 1", "lightweight boots 2", "lightweight boots 3"},
			passiveStats = {
				{-5, 0, 15, -5, 0},
				{-15, 0, 30, -15, 0},
				{-30, 0, 50, -30, 0}}
		},
		{
			name = "Cube",
			icon = "CYLCH4",
			startcolor = SKINCOLOR_SUPERCUBE1,
			skills = {"whisper_atkc", "blunt force 1", "blunt force 2", "blunt force 3", "w-rakunda", "w-marakunda", "w-fortify", "w-indigo wall", "w-indigo barrier"},
			passives = {"tank stance 1", "tank stance 2", "tank stance 3"},
			passiveStats = {
				{0, 15, -5, 0, 0, "trade", 33},
				{0, 30, -15, 0, 0, "trade", 66},
				{0, 50, -30, 0, 0, "trade", 100}}
		},
		{
			name = "Spike",
			icon = "CYLCH5",
			startcolor = SKINCOLOR_SUPERSPIKE1,
			skills = {"whisper_atks", "shrapnel shot 1", "shrapnel shot 2", "shrapnel shot 3", "w-bear trap", "w-minefield", "w-tiger trap", "w-poisma", "w-poison mist"},
			passives = {"thorned hide 1", "thorned hide 2", "thorned hide 3"},
			passiveStats = {
				{15, 0, 5, 0, 0, "thorns", 15},
				{30, 0, 10, 0, 0, "thorns", 30},
				{50, 0, 30, 0, 0, "thorns", 60}}
		}}
	for i = 1, #ogWisps
		T_WispTable[#T_WispTable+1] = ogWisps[i]
	end
end

local baseSkills = {"whisper_atkl", "wispon amp 1", "wispon amp 2", "wispon amp 3", "wispon support 1", "wispon support aoe", "wispon support strong", "wispon ability 1", "wispon ability 2"}
local basePassives = {"wispon passive 1", "wispon passive 2", "wispon passive 3"}

local pickGunSFX = function(var1, var2, var3)
	if var1 >= 40 and var2 == "\x44\x61\x6E\x74\x65"
		local sounds = {sfx_dntt, sfx_dntg, sfx_dntw, sfx_dntr, sfx_dnts}
		return sounds[var3] or sfx_whrl
	else
		return sfx_whrl
	end
end

//Skill swapping

local wispSkillExchange = function(mo, old)
	charStats["whisper"].melee_natk = T_WispTable[mo.wispwheel.currentwisp].skills[1]
	
	for i = 1, #mo.saveskills
		local skill = mo.saveskills[i]
		
		for j = 1, #T_WispTable[old].skills
			if (T_WispTable[old].skills[j] == skill) or (baseSkills[j] == skill)
				mo.saveskills[i] = T_WispTable[mo.wispwheel.currentwisp].skills[j]
			end
		end
		
		for j = 1, #T_WispTable[old].passives
			if (T_WispTable[old].passives[j] == skill) or (basePassives[j] == skill)
				mo.saveskills[i] = T_WispTable[mo.wispwheel.currentwisp].passives[j]
			end
		end
	end
	
	for i = 1, #mo.skills
		local skill = mo.skills[i]
		
		for j = 1, #T_WispTable[old].skills
			if (T_WispTable[old].skills[j] == skill) or (baseSkills[j] == skill)
				mo.skills[i] = T_WispTable[mo.wispwheel.currentwisp].skills[j]
			end
		end
	end
	
	for i = 1, #mo.passiveskills
		local skill = mo.passiveskills[i]
		
		for j = 1, #T_WispTable[old].passives
			if (T_WispTable[old].passives[j] == skill) or (basePassives[j] == skill)
				mo.passiveskills[i] = T_WispTable[mo.wispwheel.currentwisp].passives[j]
			end
		end
	end
end

addHook("MobjThinker", function(mo)
	if not mo or not mo.valid return end
	
	if server.P_BattleStatus[mo.battlen]
		local battle = server.P_BattleStatus[mo.battlen]
		local cturn = battle.turnorder[1]
		
		if battle.battlestate == BS_END
			if mo.lifted
				mo.lifted = 0
				mo.agility = mo.trueagi
				mo.trueagi = nil
			end
			
			if mo.indigowall
				mo.indigowall = 0
				mo.endurance = mo.truedef
				mo.truedef = nil
			end
			
			mo.w_spikes = 0
			mo.beartrap = 0
			mo.minetrap = 0
			mo.macetrap = 0
			
			if mo.resdown
				local elements = {ATK_STRIKE, ATK_SLASH, ATK_PIERCE, ATK_FIRE, ATK_ICE, ATK_WIND, ATK_ELEC, ATK_NUCLEAR, ATK_PSY, ATK_BLESS, ATK_CURSE}
				local bioelements
				if bionis
					bioelements = {ATK_WATER, ATK_EARTH, ATK_POISON, ATK_FOREST, ATK_METAL, ATK_GRAVITY}
				end
				
				for i = 1, #elements
					mo.buffs[elements[i]][1] = 0
					mo.buffs[elements[i]][2] = 0
				end
				if bionis
					for i = 1, #bioelements
						mo.buffs[bioelements[i]][1] = 0
						mo.buffs[bioelements[i]][2] = 0
					end
				end
				mo.resdown = 0
			end
			
			if mo.devastated
				mo.weak = mo.oldweak
				mo.oldweak = nil
				mo.devastated = 0
			end
		end
		
		//Spikes Status
		
		if mo.w_spikes and leveltime%4 == 0
			local shard = P_SpawnMobj(mo.x + P_RandomRange(-30, 30)*FRACUNIT, mo.y + P_RandomRange(-30, 30)*FRACUNIT, mo.z + mo.height + P_RandomRange(-10, 10)*FRACUNIT, MT_BUFU_PARTICLE)
			shard.state, shard.scale = S_BUFU_PARTICLE_A, mo.scale
			shard.colorized = true
			shard.color = SKINCOLOR_SUPERSPIKE3
			shard.fuse = 20
			shard.flags = MF_NOGRAVITY
			shard.momx = P_RandomRange(-6, 6)*FRACUNIT
			shard.momy = P_RandomRange(-6, 6)*FRACUNIT
			shard.momz = P_RandomRange(-4, 6)*FRACUNIT
		end
		
		//Lifted status
		
		if mo.lifted
			if mo and not mo.trueagi
				mo.trueagi = mo.agility
			end
			
			mo.agility = mo.trueagi/2
			
			if mo.hp <= 0
				mo.lifted = 0
			end
			
			if leveltime%8 == 0
				local thok = P_SpawnMobjFromMobj(mo, 0,0,
				0, MT_THOK)
				thok.renderflags = $ | RF_FULLBRIGHT | RF_NOCOLORMAPS | RF_FLOORSPRITE
				thok.blendmode = AST_ADD
				thok.color = SKINCOLOR_EMERALD
				
				thok.flags = $|MF_NOGRAVITY
				
				thok.state = S_WHISPER_CIRCLE
				thok.scale = 3*FRACUNIT/2
				thok.scalespeed = FixedMul($, FRACUNIT)
				thok.momz = FRACUNIT*5
				thok.destscale = INT32_MAX
			end
		end
		
		//Indigo Wall status
		
		if mo.indigowall
			if mo and not mo.truedef
				mo.truedef = mo.endurance
			end
			
			if mo.hp <= 0
				mo.indigowall = 0
			end
			
			mo.endurance = mo.truedef*3
			mo.tetraja = true
			
			if leveltime%4== 0
				local wall = P_SpawnMobj(mo.x + 30*cos(mo.angle), mo.y + 30*sin(mo.angle), mo.z, MT_THOK)
				wall.sprite = SPR_CARD
				wall.frame = J|FF_FULLBRIGHT|FF_PAPERSPRITE|TR_TRANS60
				wall.colorized = true
				wall.tics = 2
				wall.scale = FRACUNIT/2
				wall.angle = mo.angle + ANGLE_90
				wall.color = SKINCOLOR_COBALT
			end
		end
		
		//Special Status Effect countdowns
		
		if cturn == mo and battle.battlestate == BS_ENDTURN
			if mo.devastated
				mo.devastated = $ - 1
				if mo.devastated <= 0
					mo.weak = mo.oldweak
					mo.oldweak = nil
					mo.devastated = 0
				end
			end
			
			if mo.indigowall
				mo.indigowall = $ - 1
				
				if not mo.indigowall
					mo.endurance = mo.truedef
					mo.truedef = nil
				end
			end
				
			if mo.resdown
				mo.resdown = $ - 1
				if mo.resdown <= 0
					local elements = {ATK_STRIKE, ATK_SLASH, ATK_PIERCE, ATK_FIRE, ATK_ICE, ATK_WIND, ATK_ELEC, ATK_NUCLEAR, ATK_PSY, ATK_BLESS, ATK_CURSE}
					local bioelements
					if bionis
						bioelements = {ATK_WATER, ATK_EARTH, ATK_POISON, ATK_FOREST, ATK_METAL, ATK_GRAVITY}
					end
					
					for i = 1, #elements
						mo.buffs[elements[i]][1] = 0
						mo.buffs[elements[i]][2] = 0
					end
					if bionis
						for i = 1, #bioelements
							mo.buffs[bioelements[i]][1] = 0
							mo.buffs[bioelements[i]][2] = 0
						end
					end
					mo.resdown = 0
				end
			end
			
			if mo.lifted
				mo.lifted = $ - 1
				
				if not mo.lifted
					mo.agility = mo.trueagi
					mo.trueagi = nil
				end
			end
			
			if mo.w_spikes
				damageObject(mo, mo.maxhp/20)
				mo.w_spikes = $ - 1
			end
		end
		
		//Whisper Skill-Swap stuff
		
		if cturn == mo and mo.wispwheel
			mo.activepass = {}
			
			for i = 1, #mo.passiveskills
				for j = 1, #T_WispTable[mo.wispwheel.currentwisp].passives
					if T_WispTable[mo.wispwheel.currentwisp].passives[j] == mo.passiveskills[i]
						mo.activepass[#mo.activepass+1] = j
					end
				end
			end
			
			if not mo.truestats
				mo.truestats = {mo.strength, mo.endurance, mo.agility, mo.magic, mo.luck}
			end
			
			if mo.activepass
				local str, def, agl, mag, luk = 0, 0, 0, 0, 0
				for i = 1, #mo.activepass
					local effect = T_WispTable[mo.wispwheel.currentwisp].passiveStats[mo.activepass[i]]
					
					str = $ + effect[1]
					def = $ + effect[2]
					agl = $ + effect[3]
					mag = $ + effect[4]
					luk = $ + effect[5]
				end
				
				mo.strength = (mo.truestats[1]*(100+str))/100
				mo.endurance = (mo.truestats[2]*(100+def))/100
				mo.agility = (mo.truestats[3]*(100+agl))/100
				mo.magic = (mo.truestats[4]*(100+mag))/100
				mo.luck = (mo.truestats[5]*(100+luk))/100
				
				mo.w_thorns = 0
				mo.w_recoil = 0
				
				for i = 1, #mo.activepass
					local effect = T_WispTable[mo.wispwheel.currentwisp].passiveStats[mo.activepass[i]]
					if effect[6] -- additional effect
						if effect[6] == "recoil"
							mo.w_recoil = $ + effect[7]
						elseif effect[6] == "trade"
							local difference = (effect[7]*(mo.magic - mo.strength))/100
							mo.magic = $ - difference
							mo.strength = $ + difference
						elseif effect[6] == "thorns"
							mo.w_thorns = $ + effect[7]
						end
					end
				end
				
			end
			
			//Wisp Cooldowns and Zen Mindset
			
			if battle.battlestate == BS_ENDTURN
				//Whisper's wisp cooldowns
				
				for i = 1, #T_WispTable
					if mo.wispwheel.cooldown[i] > 0
						mo.wispwheel.cooldown[i] = $ - 1
					end
				end
			elseif battle.battlestate == BS_END
				for i = 1, #T_WispTable
					mo.wispwheel.cooldown[i] = 0
				end
			end
			
			
			if mo.zenmindset
				attackDefs["big bang 1"].type = ATK_ALMIGHTY
				attackDefs["big bang 1"].desc = "Medium Almighty dmg to all enemies. \nMay inflict burn.\nTires Orange after use."
				attackDefs["big bang 2"].type = ATK_ALMIGHTY
				attackDefs["big bang 2"].desc = "Heavy Almighty dmg to all enemies. \nMay inflict burn.\nTires Orange after use."
				attackDefs["big bang 3"].type = ATK_ALMIGHTY
				attackDefs["big bang 3"].desc = "Severe Almighty dmg to all enemies 2x. \nMay inflict burn.\nTires Orange after use."
			else
				attackDefs["big bang 1"].type = ATK_FIRE
				attackDefs["big bang 1"].desc = "Medium Fire dmg to all enemies. \nMay inflict burn.\nTires Orange after use."
				attackDefs["big bang 2"].type = ATK_FIRE
				attackDefs["big bang 2"].desc = "Heavy Fire dmg to all enemies. \nMay inflict burn.\nTires Orange after use."
				attackDefs["big bang 3"].type = ATK_FIRE
				attackDefs["big bang 3"].desc = "Severe Fire dmg to all enemies 2x. \nMay inflict burn.\nTires Orange after use."
			end
		end
			
		//Thorns/Recoil/Traps
		if not mo.lasthp and mo.hp
			mo.lasthp = mo.hp
		end
		
		if mo.hp <= 0
			mo.w_spikes = 0
		end
		
		if mo.lasthp > mo.hp and not (cturn == mo)
			local difference = mo.lasthp - mo.hp
			
			if mo.indigowall
				mo.indigowall = 0
				mo.endurance = mo.truedef
				mo.truedef = nil
			end
			
			if mo.w_thorns and mo.atk_attacker and mo.atk_attacker ~= mo
				damageObject(mo.atk_attacker, (mo.w_thorns*difference)/100)
			end
			
			if mo.atk_attacker and mo.atk_attacker.w_recoil and mo.atk_attacker ~= mo
				if (mo.atk_attacker.hp - (mo.atk_attacker.w_recoil*difference)/100) <= 0 and mo.atk_attacker.hp > 1
					damageObject(mo.atk_attacker, (mo.atk_attacker.hp - 1))
				else
					damageObject(mo.atk_attacker, (mo.atk_attacker.w_recoil*difference)/100)
				end
			end
			
			if mo.atk_attacker and mo.atk_attacker ~= mo
				if mo.macetrap
					local target = mo.atk_attacker
					
					playSound(mo.battlen, sfx_bufu6)
					createSplat(mo)
					createSplat(mo)
					localquake(mo.battlen, FRACUNIT*32, 10)
					damageObject(target, difference)
					for i = 1, 32 do
						local p = P_SpawnMobj(target.x, target.y, target.z+FRACUNIT*20, MT_DUMMY)
						p.flags = MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY
						p.state = S_SSPK1
						p.scale = mo.scale
						p.colorized = true
						p.color = SKINCOLOR_SUPERSPIKE3
						p.angle = target.angle + P_RandomRange(-30, 30)*ANG1
						p.momz = P_RandomRange(-24, 24)*mo.scale
						P_InstaThrust(p, p.angle, mo.scale*64)
					end

					local d = P_SpawnMobj(target.x, target.y, target.z+128*FRACUNIT, MT_THOK)
					d.sprite = SPR_THMP
					d.frame = A|FF_FULLBRIGHT
					d.scale = FRACUNIT/4
					d.destscale = FRACUNIT*6
					d.scalespeed = FRACUNIT/8
					d.fuse = TICRATE-10
					d.tics = d.fuse
					d.momz = FRACUNIT*2
					d.momx = P_RandomRange(0, FRACUNIT-1)*P_RandomRange(-8, 8)
					d.momy = P_RandomRange(0, FRACUNIT-1)*P_RandomRange(-8, 8)
					d.color = SKINCOLOR_SUPERSPIKE3
					BTL_logMessage(mo.battlen, "Tiger trap triggered!")
					mo.macetrap = false
				elseif mo.beartrap
					BTL_logMessage(mo.battlen, "Spike trap triggered!")
					damageObject(mo.atk_attacker, (50*difference)/100)
					mo.beartrap = false
					playSound(mo.battlen, sfx_spkdth)
				elseif mo.minetrap
					DoExplosion(mo.atk_attacker)
					
					BTL_logMessage(mo.battlen, "Mine trap triggered!")
					damageObject(mo.atk_attacker, (40*difference)/100)
					mo.minetrap = false
				end
			end
		end
		
		mo.lasthp = mo.hp
	end
end, MT_PFIGHTER)

addHook("PlayerThink", function(p)
	local b = p.control
	if not b or not b.valid or not b.battlen return end
	local battle = server.P_BattleStatus[b.battlen]
	local mo = battle.turnorder[1]
	if battle.battlestate ~= BS_DOTURN or not (mo.plyr and mo.control)
		if mo and mo.valid and mo.wispwheel
			mo.wispwheel.clicks = 0
		end
		
		return
	end
	
	local buffer = (-45) + (360/#T_WispTable)
	
	if not mo or not mo.valid or mo.t_act ~= ACT_NONE or mo.skin ~= "whisper"  return end
	
	mo.wispwheel = $ or {
	currentwisp = $ or 1,
	rotating = $ or 0,
	direction = false,
	clicks = $ or 0,
	angle = $ or buffer*ANG1}
	
	if not mo.wispwheel.cooldown
		mo.wispwheel.cooldown = {}
		for i = 1, #T_WispTable
			mo.wispwheel.cooldown[i] = 0
		end
	end
	
	local oldwisp = mo.wispwheel.currentwisp
	
	if (mo.control == consoleplayer) and not p.buttonheld and not mo.wispwheel.rotating
		if p.cmd.buttons & BT_CUSTOM2
			mo.wispwheel.currentwisp = $%#T_WispTable + 1
			mo.wispwheel.direction = "cw"
			mo.wispwheel.rotating = 6
			mo.wispwheel.clicks = $ + 1
			p.buttonheld = true
			playSound(mo.battlen, pickGunSFX(mo.wispwheel.clicks, mo.name, mo.wispwheel.currentwisp))
		elseif p.cmd.buttons & BT_CUSTOM3
			mo.wispwheel.currentwisp = ($+(#T_WispTable-2))%#T_WispTable + 1
			mo.wispwheel.direction = "ccw"
			mo.wispwheel.rotating = 6
			mo.wispwheel.clicks = $ + 1
			p.buttonheld = true
			playSound(mo.battlen, pickGunSFX(mo.wispwheel.clicks, mo.name, mo.wispwheel.currentwisp))
		end
	elseif (mo.control == consoleplayer) and not mo.wispwheel.rotating and not (p.cmd.buttons & BT_CUSTOM2 or p.cmd.buttons & BT_CUSTOM3)
		p.buttonheld = false
	end
	
	wispSkillExchange(mo, oldwisp)
	
	if mo.wispwheel.rotating
		local change = 360/#T_WispTable
		if mo.wispwheel.direction == "cw"
			mo.wispwheel.angle = $ - (ANG1*change)/6
			mo.wispwheel.rotating = $ - 1
		elseif mo.wispwheel.direction == "ccw"
			mo.wispwheel.angle = $ + (ANG1*change)/6
			mo.wispwheel.rotating = $ - 1
		else
			mo.wispwheel.direction = false
		end
	end
	
end)

local drawWispWheel = function(v, p, c, mo)
	v.originx = $ or -83*FRACUNIT
	v.originy = $ or 0
	local change = 360/#T_WispTable
	
	if not mo or not mo.valid or not mo.wispwheel or mo.skin ~= "whisper" return end
	
	if mo.t_act == ACT_NONE
		v.originx = (v.originx < -3*FRACUNIT and ($ + 20*FRACUNIT) or $)
	else
		v.originx = (v.originx > -83*FRACUNIT and ($ - 20*FRACUNIT) or $)
	end
	
	local cylinder, flip
	local wheel = v.cachePatch("WHCYL1")
	cylinder, flip = v.getSpritePatch("WHCY", A, 0, mo.wispwheel.angle)
	
	local colindex = abs((leveltime/3)%8 - 4)
	local color = v.getColormap(TC_DEFAULT, T_WispTable[mo.wispwheel.currentwisp].startcolor + colindex)
	v.drawScaled(v.originx, v.originy, 3*FRACUNIT/4, wheel, V_SNAPTOTOP|V_SNAPTOLEFT, color) -- first, draw the gradient
	v.drawScaled(v.originx, v.originy, 3*FRACUNIT/4, cylinder, V_SNAPTOTOP|V_SNAPTOLEFT) -- next, draw the cylinder
	
	local x, y, ang, wisp
	local greymap = v.getColormap(TC_RAINBOW, SKINCOLOR_GREY)
	
	x = v.originx + 48*cos(ANG1*change - ANG1* ((-45) + (360/#T_WispTable)))
	y = v.originy + 48*sin(ANG1*change - ANG1* ((-45) + (360/#T_WispTable)))
    local chamber = v.cachePatch("CYLCHB")
	local chcolor = v.getColormap(TC_DEFAULT, T_WispTable[mo.wispwheel.currentwisp].startcolor+4)
	v.drawScaled(x, y, (4*FRACUNIT)/3, chamber, V_SNAPTOTOP|V_SNAPTOLEFT, chcolor)
	
	for i = 1,#T_WispTable
		ang = (ANG1*change)*(i-1)
		x = v.originx + 48*cos(ANG1*change-ang-mo.wispwheel.angle)
		y = v.originy + 48*sin(ANG1*change-ang-mo.wispwheel.angle)
		
		wisp = v.cachePatch(T_WispTable[i].icon)
		if not mo.wispwheel.cooldown[i]
			v.drawScaled(x, y, (4*FRACUNIT)/3, wisp, V_SNAPTOTOP|V_SNAPTOLEFT)
		else
			v.drawScaled(x, y, (4*FRACUNIT)/3, wisp, V_SNAPTOTOP|V_SNAPTOLEFT, greymap)
			
			if mo.wispwheel.cooldown[i] <= 5
				local cooldown = v.cachePatch("CYCLD"..mo.wispwheel.cooldown[i])
				v.drawScaled(x, y, (4*FRACUNIT)/3, cooldown, V_SNAPTOTOP|V_SNAPTOLEFT)
			end
		end
		
		if mo.wispwheel.currentwisp == i and not mo.wispwheel.cooldown[i]
			local circle
			circle, flip = v.getSpritePatch("WSPT", A, 0, 0)
			v.drawScaled(x, y-3*FRACUNIT, ((leveltime%20+10)*FRACUNIT)/30, circle, V_60TRANS|V_SNAPTOTOP|V_SNAPTOLEFT, (v.getColormap(TC_DEFAULT, T_WispTable[mo.wispwheel.currentwisp].startcolor)))
		end
	end
	
	if mo.w_recoil
		v.drawString(3,90,"\x87Recoil\x80 Active!",V_SNAPTOTOP|V_SNAPTOLEFT, "thin")
		if mo.w_thorns
			v.drawString(3,105,"\x81Thorns\x80 Active!",V_SNAPTOTOP|V_SNAPTOLEFT, "thin")
		end
	elseif mo.w_thorns
		v.drawString(3,90,"\x81Thorns\x80 Active!",V_SNAPTOTOP|V_SNAPTOLEFT, "thin")
	end
	
	if v.originx >= -3*FRACUNIT
		v.drawString(75,3,"C2",V_SNAPTOTOP|V_SNAPTOLEFT, "thin")
		v.drawString(3,75,"C3",V_SNAPTOTOP|V_SNAPTOLEFT, "thin")
	end
end

hud.add(function(v, p, c)
	local b = p.control
	if not b or not b.valid or not b.battlen return end
	
	local battle = server.P_BattleStatus[b.battlen]
	local cturn = battle.turnorder[1]
	if battle.battlestate == BS_DOTURN and cturn.plyr and cturn.control
		if server.P_BattleStatus[b.battlen]
		and server.P_BattleStatus[b.battlen].running
		and cturn and cturn.valid
			drawWispWheel(v, p, c, cturn)
		end
	else
		v.originx = -83*FRACUNIT
	end
end)